import 'dart:io';

import 'package:morpheme_cli/constants.dart';
import 'package:morpheme_cli/dependency_manager.dart';

class ChangelogCommand extends Command {
  ChangelogCommand() {
    argParser.addOption(
      'path',
      help: 'Path changelog file',
    );
  }

  @override
  String get name => 'changelog';

  @override
  String get description => 'get latest changelog';

  @override
  String get category => Constants.tools;

  String getLatestChangelog(String changelog) {
    final regex = RegExp(
        r'(#+)(\s+)?(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?');
    final match = regex.allMatches(changelog).take(2).toList();
    if (match.length == 2) {
      return changelog
          .substring(match[0].start, match[1].start)
          .replaceAll(regex, '')
          .trim();
    } else {
      return changelog.replaceAll(regex, '').trim();
    }
  }

  @override
  void run() {
    final pathArgs = argResults?['path'] ?? 'CHANGELOG.md';
    String changelog = '';
    final path = join(current, pathArgs);

    if (exists(join(current, path))) {
      changelog = File(path).readAsStringSync();
    }
    print(getLatestChangelog(changelog));
  }
}
